package de.ozgcloud.nachrichten.postfach.osiv2.transfer;

import java.time.OffsetDateTime;
import java.time.ZonedDateTime;
import java.util.UUID;

import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Named;
import org.mapstruct.ReportingPolicy;

import de.ozgcloud.nachrichten.postfach.PostfachAddress;
import de.ozgcloud.nachrichten.postfach.PostfachAddressIdentifier;
import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.osiv2.OsiPostfachRemoteService;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.V1ReplyBehavior;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.V1ReplyMessage;
import lombok.Builder;
import lombok.Getter;

@Mapper(unmappedTargetPolicy = ReportingPolicy.ERROR, imports = { Osi2HtmlDocument.class })
public interface Osi2ResponseMapper {

	String POSTFACH_ADDRESS_VERSION = "2.0";
	int POSTFACH_ADDRESS_TYPE = 2;

	@Mapping(target = "id", ignore = true)
	@Mapping(target = "vorgangId", source = "sequencenumber")
	@Mapping(target = "postfachAddress", source = "messageBox")
	@Mapping(target = "messageId", source = "guid")
	@Mapping(target = "referencedNachricht", ignore = true)

	@Mapping(target = "createdAt", source = "responseTime", qualifiedByName = "mapOffsetDateTimeToZoned")
	@Mapping(target = "createdBy", ignore = true)

	@Mapping(target = "sentAt", ignore = true)
	@Mapping(target = "sentSuccessful", ignore = true)
	@Mapping(target = "messageCode", ignore = true)

	@Mapping(target = "direction", constant = "IN")

	@Mapping(target = "subject", source = "subject")
	@Mapping(target = "mailBody", source = ".", qualifiedByName = "mapMailBody")
	@Mapping(target = "replyOption", source = "replyAction")
	@Mapping(target = "attachments", ignore = true)
	PostfachNachricht toPostfachNachricht(V1ReplyMessage message);

	default String mapNullToEmpty(String value) {
		return value == null ? "" : value;
	}

	@Named("mapOffsetDateTimeToZoned")
	default ZonedDateTime mapOffsetDateTimeToZoned(OffsetDateTime offsetDateTime) {
		return offsetDateTime == null ? null : offsetDateTime.toZonedDateTime();
	}

	@Named("mapMailBody")
	default String mapMailBody(V1ReplyMessage message) {
		var body = mapNullToEmpty(message.getBody());
		return Boolean.TRUE.equals(message.getIsHtml())
				? Osi2HtmlDocument.renderToPlainText(body)
				: body;
	}

	default PostfachAddress buildPostfachAddressByPostfachId(UUID messageBox) {
		return messageBox == null
				? null
				: PostfachAddress.builder()
						.type(POSTFACH_ADDRESS_TYPE)
						.version(POSTFACH_ADDRESS_VERSION)
						.identifier(StringBasedIdentifier.builder()
								.mailboxId(messageBox.toString())
								.build())
						.serviceKontoType(OsiPostfachRemoteService.POSTFACH_TYPE_OSI)
						.build();

	}

	default PostfachNachricht.ReplyOption mapReplyAction(V1ReplyBehavior replyOption) {
		return replyOption == null
				? PostfachNachricht.ReplyOption.FORBIDDEN
				: switch (replyOption) {
					case REPLYFORBIDDEN -> PostfachNachricht.ReplyOption.FORBIDDEN;
					case REPLYPOSSIBLE -> PostfachNachricht.ReplyOption.POSSIBLE;
					case REPLYMANDATORY -> PostfachNachricht.ReplyOption.MANDATORY;
				};
	}

	@Builder
	@Getter
	class StringBasedIdentifier implements PostfachAddressIdentifier {

		private String mailboxId;

		@Override
		public boolean isStringBasedIdentifier() {
			return true;
		}

		@Override
		public String toString() {
			return mailboxId;
		}
	}
}
